==> trircd-release-5.1.7-r.tar.gz: trircd-release-5.1.7-r/modules/m_rping.c <==
/************************************************************************
 *   IRC - Internet Relay Chat, modules/m_ping.c
 *
 *   Copyright (C) 2000-2002 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free softwmare; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* $Id: m_rping.c,v 5.7 2003/03/14 11:44:18 tr-ircd Exp $ */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"

static char *token = TOK1_RPING;
static char *atoken = TOK1_RPONG;

static struct Message _msgtab[] = {
    {MSG_RPING, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, u_rping, o_rping, s_rping, m_ignore}
};
static struct Message n_msgtab[] = {
    {MSG_RPONG, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_ignore, m_ignore, m_rpong, m_ignore}
};

#ifndef STATIC_MODULES

char *_version = "$Revision: 5.7 $";

void _modinit(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
    mod_add_cmd(n_msgtab);
    tok1_msgtab[(u_char) *atoken].msg = n_msgtab;
}

void _moddeinit(void)
{
    mod_del_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = NULL;
    mod_del_cmd(n_msgtab);
    tok1_msgtab[(u_char) *atoken].msg = NULL;
}
#else
void m_rping_init(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
    mod_add_cmd(n_msgtab);
    tok1_msgtab[(u_char) *atoken].msg = n_msgtab;
}

#endif

int u_rping(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    aClient *acptr;

    if (parc < 2) {
	send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_RPING);
	return 0;
    }

    if (parc < 3) {
	if (irc_strcmp(parv[1], me.name) == 0) {
	    send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_RPING);
	    return 0;
	}
	acptr = find_server(parv[1]);
	if (!acptr) {
	    send_me_numeric(sptr, ERR_NOSUCHSERVER, parv[1]);
            return 0;
        }
	sendto_one_server(acptr, &me, TOK1_RPONG, "%~C %s :%lu", sptr, parv[1], irc_time(NULL));
	return 0;	
    } else {
	send_me_numeric(sptr, ERR_NOPRIVILEGES);
    }

    return 0;
}

int o_rping(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    aClient *acptr;
    aClient *bcptr;

    if (parc < 2) {
        send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_RPING);
        return 0;
    }

    acptr = find_server(parv[1]);
    if (!acptr) {
        send_me_numeric(sptr, ERR_NOSUCHSERVER, parv[1]);
        return 0;
    }

    if (parc < 3 || (irc_strcmp(parv[1],me.name) == 0)) {
        if ((parc < 3 && (irc_strcmp(parv[1], me.name) == 0)) 
         || (parc > 2 && (irc_strcmp(parv[1], parv[2]) == 0))) {
            send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_RPING);
            return 0;
        }
	else if (parc > 2)
	    acptr = find_server(parv[2]);
	if (!acptr) {
	    send_me_numeric(sptr, ERR_NOSUCHSERVER, parv[2]);
	    return 0;
	}
	sendto_one_server(acptr, &me, TOK1_RPONG, "%~C %C :%lu", sptr, acptr, irc_time(NULL));
        return 0;
    }
    
    bcptr = find_server(parv[2]);
    if (!bcptr) {
        send_me_numeric(sptr, ERR_NOSUCHSERVER, parv[2]);
        return 0;
    }

    sendto_one_server(acptr, &me, TOK1_RPING, "%~C %C %C", sptr, bcptr, acptr);
    return 0;
}

int s_rping(aClient *cptr, aClient *sptr, int parc, char *parv[])
{ 
    aClient *acptr;

    if (parc < 4)
	return 0;

    if (hunt_server(cptr, sptr, ":%s %s %s %s %s", TOK1_RPING, 3, parc, parv) != HUNTED_ISME) {
	return 0;
    } 
    else {
	acptr = find_server(parv[2]);
        if (!acptr) {
	    acptr = find_client(parv[1]);
            if (!acptr) 
            	return 0;
            send_me_numeric(acptr, ERR_NOSUCHSERVER, parv[2]);
            return 0;
        }
        sendto_one_server(acptr, &me, TOK1_RPONG, "%s %s :%lu", parv[1], parv[2], irc_time(NULL));
    }

    return 0;
}


int m_rpong(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    aClient *acptr;

    if (parc < 4) 
	return 0;

    if (hunt_server(cptr, sptr, ":%s %s %s %s %s", TOK1_RPONG, 2, parc, parv) != HUNTED_ISME) {
	return 0;
    } 

    acptr = find_client(parv[1]);

    if (!acptr) 
	return 0;

    send_me_numeric(acptr, RPL_RPONG, sptr->name, me.name, irc_time(NULL) - atol(parv[3]));
    return 0;
}

==> trircd-release-5.7.3-r.tar.gz: trircd-release-5.7.3-r/modules/m_rping.c <==
/************************************************************************
 *   IRC - Internet Relay Chat, modules/m_ping.c
 *
 *   Copyright (C) 2000-2003 TR-IRCD Development
 *
 *   Copyright (C) 1990 Jarkko Oikarinen and
 *                      University of Oulu, Co Center
 *
 *   See file AUTHORS in IRC package for additional names of
 *   the programmers.
 *
 *   This program is free softwmare; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* $Id: m_rping.c,v 1.4 2004/02/24 15:00:27 tr-ircd Exp $ */

#include "struct.h"
#include "common.h"
#include "sys.h"
#include "numeric.h"
#include "msg.h"
#include "channel.h"
#include "h.h"

static char *token = TOK1_RPING;
static char *atoken = TOK1_RPONG;

static struct Message _msgtab[] = {
    {MSG_RPING, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, u_rping, o_rping, s_rping, m_ignore}
};
static struct Message n_msgtab[] = {
    {MSG_RPONG, 0, MAXPARA, M_SLOW, 0L,
     m_unregistered, m_ignore, m_ignore, m_rpong, m_ignore}
};

#ifndef STATIC_MODULES

char *_version = "$Revision: 1.4 $";

void _modinit(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
    mod_add_cmd(n_msgtab);
    tok1_msgtab[(u_char) *atoken].msg = n_msgtab;
}

void _moddeinit(void)
{
    mod_del_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = NULL;
    mod_del_cmd(n_msgtab);
    tok1_msgtab[(u_char) *atoken].msg = NULL;
}
#else
void m_rping_init(void)
{
    mod_add_cmd(_msgtab);
    tok1_msgtab[(u_char) *token].msg = _msgtab;
    mod_add_cmd(n_msgtab);
    tok1_msgtab[(u_char) *atoken].msg = n_msgtab;
}

#endif

int u_rping(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    aClient *acptr;

    if (parc < 2) {
	send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_RPING);
	return 0;
    }

    if (parc < 3) {
	if (strcmp(parv[1], me.name) == 0) {
	    send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_RPING);
	    return 0;
	}
	acptr = find_server(parv[1]);
	if (!acptr) {
	    send_me_numeric(sptr, ERR_NOSUCHSERVER, parv[1]);
            return 0;
        }
	sendto_one_server(acptr, &me, TOK1_RPONG, "%~C %s :%lu", sptr, parv[1], time(NULL));
	return 0;	
    } else {
	send_me_numeric(sptr, ERR_NOPRIVILEGES);
    }

    return 0;
}

int o_rping(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    aClient *acptr;
    aClient *bcptr;

    if (parc < 2) {
        send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_RPING);
        return 0;
    }

    acptr = find_server(parv[1]);
    if (!acptr) {
        send_me_numeric(sptr, ERR_NOSUCHSERVER, parv[1]);
        return 0;
    }

    if (parc < 3 || (strcmp(parv[1],me.name) == 0)) {
        if ((parc < 3 && (strcmp(parv[1], me.name) == 0)) 
         || (parc > 2 && (strcmp(parv[1], parv[2]) == 0))) {
            send_me_numeric(sptr, ERR_NEEDMOREPARAMS, MSG_RPING);
            return 0;
        }
	else if (parc > 2)
	    acptr = find_server(parv[2]);
        if (!acptr) {
            send_me_numeric(sptr, ERR_NOSUCHSERVER, parv[2]);
            return 0;
        }
	sendto_one_server(acptr, &me, TOK1_RPONG, "%~C %C :%lu", sptr, acptr, time(NULL));
        return 0;
    }
    
    bcptr = find_server(parv[2]);
    if (!bcptr) {
        send_me_numeric(sptr, ERR_NOSUCHSERVER, parv[2]);
        return 0;
    }

    sendto_one_server(acptr, &me, TOK1_RPING, "%~C %C %C", sptr, bcptr, acptr);
    return 0;
}

int s_rping(aClient *cptr, aClient *sptr, int parc, char *parv[])
{ 
    aClient *acptr;

    if (parc < 4)
	return 0;

    if (hunt_server(cptr, sptr, ":%s %s %s %s %s", TOK1_RPING, 3, parc, parv) != HUNTED_ISME) {
	return 0;
    } 
    else {
	acptr = find_server(parv[2]);
        if (!acptr) {
	    acptr = find_client(parv[1]);
            if (!acptr) 
            	return 0;
            send_me_numeric(acptr, ERR_NOSUCHSERVER, parv[2]);
            return 0;
        }
        sendto_one_server(acptr, &me, TOK1_RPONG, "%s %s :%lu", parv[1], parv[2], time(NULL));
    }

    return 0;
}


int m_rpong(aClient *cptr, aClient *sptr, int parc, char *parv[])
{
    aClient *acptr;

    if (parc < 4) 
	return 0;

    if (hunt_server(cptr, sptr, ":%s %s %s %s %s", TOK1_RPONG, 2, parc, parv) != HUNTED_ISME) {
	return 0;
    } 

    acptr = find_client(parv[1]);

    if (!acptr) 
	return 0;

    send_me_numeric(acptr, RPL_RPONG, sptr->name, me.name, time(NULL) - atol(parv[3]));
    return 0;
}
